/** @jsxImportSource sigl */ "use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    languages: ()=>_codeSyntax.languages,
    CodeEditElement: ()=>CodeEditElement
});
const _jsxRuntime = require("sigl/jsx-runtime");
const _sigl = /*#__PURE__*/ _interopRequireDefault(require("sigl"));
const _getElementOffset = require("get-element-offset");
const _relativeMouse = require("relative-mouse");
const _codeSyntax = require("code-syntax");
const _superImpose = require("super-impose");
const _textareaCode = require("textarea-code");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
var __decorate = (void 0) && (void 0).__decorate || function(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for(var i = decorators.length - 1; i >= 0; i--)if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (void 0) && (void 0).__metadata || function(k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const style = ({ fontSize =9.5  })=>/*css*/ `
:host {
  contain: size layout style paint;
  --caret: #fff;
  --selection: #742;
  --padding: 0;
  --font-size: ${fontSize}pt;
  position: relative;
  font-size: var(--font-size);
  display: block;
  overflow: hidden;
  background: var(--background, transparent);
}
slot {
  display: none;
}
pre,
textarea {
  contain: layout style paint;
  z-index: 3;
  display: flex;
  white-space: pre;
  padding: 0;
  padding-right: 10px;
  margin: 0;
  width: 100%;
  height: 100%;
  resize: none;
  outline: none;
  caret-color: transparent;
  background: transparent;
  color: transparent;
  border: none;
  scroll-behavior: smooth;
}
pre {
  left: 0;
  top: 0;
  position: absolute;
  width: auto;
  height: auto;
  z-index: -1;
}
pre,
textarea,
code {
  shape-rendering: optimizeSpeed;
  text-rendering: optimizeSpeed;
  font-size: inherit;
  font-family: inherit;
  line-height: inherit;
}
textarea::selection {
  color: transparent;
  background-color: var(--selection) !important;
}
.shadow {
  text-rendering: optimizeSpeed;
  contain: layout style paint;
  position: absolute;
  color: #000;
  /* font-weight: bold; */
  z-index: 1;
}
[part=syntax] {
  text-rendering: optimizeSpeed;
  position: absolute;
  background: transparent;
  z-index: 2;
}
[part=lenses],
[part=markers] {
  contain: size layout style;
  text-rendering: optimizeSpeed;
  position: absolute;
  left: 0;
  top: 0;
  z-index: -1;
}
[part=markers] .marker {
  display: inline;
  color: transparent;
}
[part=lenses] .lens {
  display: inline-block;
  color: var(--brightBlack);
  font-style: italic;
  transform: scale(0.8);
}
[part=layout] {
  overflow: hidden !important;
}
[part=layout]::part(scroller) {
  transition: all 65ms ease-out;
}
@keyframes blink {
  from,
  to {
    opacity: 0.0;
  }
  50% {
    opacity: 1.0;
  }
}
/* .caret {
  z-index: 1;
  pointer-events: none;
} */
[part=caret] {
  height: 1em;
  position: absolute;
  width: calc(var(--font-size) * 0.08);
  background: var(--caret);
  margin-left: 0.5px;
  opacity: 0.3;
}
:host([hasfocus]) [part=caret] {
  opacity: 1;
}
:host([hasfocus][caretblink=blink]) [part=caret] {
  animation: blink 1.2s step-end infinite;
}
/* :host([caretshift=forward]) [part=caret] {
  margin-left: 0;
} */
:host([caretshift=backward]) [part=caret] {
  margin-left: calc(var(--font-size) * -0.08);;
}
`;
let CodeEditElement = class CodeEditElement extends HTMLElement {
    /** The editor's value */ value = '';
    /** Font size */ fontSize = 9.5;
    /** Autofocus to the element when page loads */ autoFocus = false;
    /** Whether to autoresize to the height of the contents */ autoResize = false;
    /** Language for syntax highlighting */ language = '';
    /** Theme to use */ theme = '';
    /** Tab size */ tabSize = 2;
    /** Tab style */ tabStyle = 'spaces';
    /** Comment */ comments = '// /* */';
    /** Shadow */ textShadow = false;
    hasFocus = false;
    CodeSyntax = _sigl.default.element(_codeSyntax.CodeSyntaxElement);
    SuperImpose = _sigl.default.element(_superImpose.SuperImposeElement);
    TextAreaCode = _sigl.default.element(_textareaCode.TextAreaCodeElement);
    TextShadow;
    /** Passing a syntax definition regexp manually */ syntax = {};
    textarea;
    layout;
    height;
    /** Custom markers */ markers = [];
    markersCss = ()=>'';
    hoveringMarker;
    /** Lens */ lenses = [];
    root = (0, _sigl.default)(this).shadow();
    get scrollLeft() {
        return this.layout?.scrollLeft ?? 0;
    }
    get scrollTop() {
        return this.layout?.scrollTop ?? 0;
    }
    onInput = (0, _sigl.default)(this).reduce(({ $ , textarea  })=>()=>{
            const { scrollLeft , scrollTop  } = this;
            $.value = textarea.value;
            Object.assign(textarea, {
                scrollLeft,
                scrollTop
            });
        }, ()=>{});
    onScroll = (0, _sigl.default)(this).reduce(({ host  })=>()=>{
            _sigl.default.dispatch.composed(host, 'scroll');
        }, ()=>{});
    onWheel = (0, _sigl.default)(this).reduce(({ $  })=>$.event.not.passive((e)=>{
            if (e.ctrlKey || e.metaKey) {
                e.stopImmediatePropagation();
                e.preventDefault();
                $.fontSize = Math.max(1, $.fontSize + Math.sign(e.deltaY) * 0.5);
            }
        }));
    caretIndex = 0;
    caretShift = 'none';
    caretBlink = 'none';
    caretBlinkTimeout = -1;
    updateCaret = (0, _sigl.default)(this).reduce(({ $ , textarea  })=>()=>{
            const range = textarea.buffer?.getRange();
            if (range) {
                if (range.hasSelection) {
                    $.caretShift = range.selectionDirection;
                } else {
                    $.caretShift = 'none';
                }
                $.caretBlink = 'none';
                $.caretIndex = range.caretIndex ?? 0;
                clearTimeout($.caretBlinkTimeout);
                $.caretBlinkTimeout = setTimeout(()=>{
                    $.caretBlink = 'blink';
                }, 500);
            }
        });
    mounted($) {
        $.effect(({ host , textarea  })=>{
            host.tabIndex = 0;
            return $.chain($.on(textarea).pointerdown.capture.stop(()=>{
                $.hasFocus = true;
            }), $.on(host).focus((e)=>{
                $.hasFocus = true;
                if (!e.composedPath().includes(textarea)) {
                    textarea.focus({
                        preventScroll: true
                    });
                }
            }), $.on(textarea).blur(()=>{
                $.hasFocus = false;
                // NOTE: without this, an invisible range can stay inside
                // the textarea that causes problems on refocus
                // e.g you can accidentally drag a selection range and move
                // the text while trying to focus (happens often).
                // The expectation when clicking outside a code editor (blurring)
                // that ranges are also removed, but that isn't the default
                // behavior in the browser. So we remove the ranges forcibly here.
                // setTimeout(() => {
                // we did actually blur and not refocus
                // if ($.getActiveElement(host) !== textarea) {
                window.getSelection()?.removeAllRanges();
            // }
            // }, 100)
            }));
        });
        $.effect(({ host , textarea , updateCaret  })=>$.chain($.on(document).selectionchange.task(()=>{
                if ($.getActiveElement(host) === textarea) {
                    updateCaret();
                }
            }), $.on(textarea).input(updateCaret)));
        $.effect(({ root  })=>$.onTextChange(root, (text)=>{
                $.value = text;
            }));
        $.effect(({ textarea  })=>{
            setTimeout(()=>{
                textarea.setSelectionRange(0, 0, 'forward');
            // arbitrary delay to move caret to the beginning initially.
            // increase to fix issues with caret being at the end initially.
            }, 100);
        });
        $.effect(({ autoResize , layout , height , value: _  })=>{
            if (autoResize) {
                // push to the end of the job when height has actually changed
                queueMicrotask(()=>{
                    layout.style.height = (height.offsetHeight ?? 'auto') + 'px';
                });
            }
        });
        $.effect(({ autoFocus , textarea  })=>{
            if (autoFocus) {
                // deal with browser autofocus warning on initial load
                setTimeout(()=>{
                    textarea.focus();
                    // move the caret to the beginning
                    setTimeout(()=>textarea.setSelectionRange(0, 0, 'forward'));
                });
            }
        });
        $.effect(({ host  })=>$.on(host).pointerleave(()=>{
                $.hoveringMarker = null;
            }));
        $.effect(({ host , hoveringMarker  })=>{
            const el1 = hoveringMarker.el;
            hoveringMarker.onenter?.(el1);
            el1.classList.add('hover');
            const offWheel = $.on(host).wheel.not.passive((ev)=>{
                hoveringMarker.onwheel?.(ev);
            });
            return ()=>{
                hoveringMarker.onleave?.(el1);
                el1.classList.remove('hover');
                return offWheel();
            };
        });
        $.effect(({ textarea , markers , fontSize: _  })=>$.on(textarea).pointermove((e)=>{
                const pos = (0, _relativeMouse.getRelativeMouse)(textarea, {
                    x: e.offsetX,
                    y: e.offsetY
                });
                const offset = (0, _getElementOffset.getElementOffset)(textarea);
                const x = pos.x + offset.x + textarea.scrollLeft;
                const y = pos.y + offset.y + textarea.scrollTop;
                for (const marker of markers){
                    const rect = marker.rect;
                    if (x > rect.left && x < rect.right && y > rect.top && y < rect.bottom) {
                        $.hoveringMarker = marker;
                        return;
                    }
                }
                $.hoveringMarker = null;
            }));
        const Markers = $.part(({ markers , markersCss , fontSize: _  })=>/*#__PURE__*/ (0, _jsxRuntime.jsxs)("div", {
                part: "markers",
                children: [
                    /*#__PURE__*/ (0, _jsxRuntime.jsx)("style", {
                        children: markersCss('[part=markers]')
                    }),
                    markers.map((x)=>{
                        const start = x.index;
                        const end = x.index + x.size;
                        return /*#__PURE__*/ (0, _jsxRuntime.jsx)("pre", {
                            children: /*#__PURE__*/ (0, _jsxRuntime.jsxs)("code", {
                                children: [
                                    x.source.slice(0, start),
                                    /*#__PURE__*/ (0, _jsxRuntime.jsx)("span", {
                                        class: [
                                            'marker',
                                            x.class
                                        ].join(' '),
                                        ref: {
                                            get current () {
                                                return x.el;
                                            },
                                            set current (el){
                                                x.el = el;
                                                if (!x.rect || x.rect.left !== el.offsetLeft || x.rect.top !== el.offsetTop) {
                                                    x.rect = new DOMRect(el.offsetLeft, el.offsetTop, el.offsetWidth, el.offsetHeight);
                                                }
                                                x.oncreate?.(el);
                                            }
                                        },
                                        children: x.source.slice(start, end)
                                    }),
                                    x.source.slice(end)
                                ]
                            })
                        }, x.key);
                    })
                ]
            }));
        const Lenses = $.part(({ lenses  })=>{
            const lines = $.value.split('\n');
            return /*#__PURE__*/ (0, _jsxRuntime.jsx)("div", {
                part: "lenses",
                children: /*#__PURE__*/ (0, _jsxRuntime.jsx)("pre", {
                    children: /*#__PURE__*/ (0, _jsxRuntime.jsx)("code", {
                        children: lines.map((text, line)=>/*#__PURE__*/ (0, _jsxRuntime.jsxs)(_jsxRuntime.Fragment, {
                                children: [
                                    text,
                                    lenses.filter((x)=>x.line - 1 == line).map((x)=>/*#__PURE__*/ (0, _jsxRuntime.jsx)("span", {
                                            class: "lens",
                                            children: x.message
                                        })),
                                    '\n'
                                ]
                            }))
                    })
                })
            });
        });
        $.TextShadow = $.reduce(({ value  })=>({ x , y  })=>/*#__PURE__*/ (0, _jsxRuntime.jsx)("pre", {
                    class: "shadow",
                    style: `left: ${x}px; top: ${y}px;`,
                    children: /*#__PURE__*/ (0, _jsxRuntime.jsx)("code", {
                        children: value
                    })
                }));
        const Caret = $.part(({ value , caretIndex  })=>/*#__PURE__*/ (0, _jsxRuntime.jsx)("pre", {
                class: "caret",
                children: /*#__PURE__*/ (0, _jsxRuntime.jsxs)("code", {
                    children: [
                        value.slice(0, caretIndex),
                        /*#__PURE__*/ (0, _jsxRuntime.jsx)("span", {
                            part: "caret"
                        }),
                        value.slice(caretIndex),
                        ' '
                    ]
                })
            }));
        const Shadow = $.part(({ TextShadow  })=>[
                -0.5,
                0,
                0.5
            ].flatMap((x)=>[
                    -0.5,
                    0,
                    0.5
                ].map((y)=>/*#__PURE__*/ (0, _jsxRuntime.jsx)(TextShadow, {
                        x: x,
                        y: y
                    }))));
        const Content = $.part(({ SuperImpose , TextAreaCode , CodeSyntax , onWheel , onScroll , onInput , comments , tabSize , tabStyle , theme , language , textShadow , syntax , value ,  })=>/*#__PURE__*/ (0, _jsxRuntime.jsxs)(_jsxRuntime.Fragment, {
                children: [
                    /*#__PURE__*/ (0, _jsxRuntime.jsxs)("pre", {
                        ref: $.ref.height,
                        children: [
                            value,
                            ' '
                        ]
                    }),
                    /*#__PURE__*/ (0, _jsxRuntime.jsxs)(SuperImpose, {
                        part: "layout",
                        ref: $.ref.layout,
                        onwheel: onWheel,
                        onscroll: onScroll,
                        children: [
                            /*#__PURE__*/ (0, _jsxRuntime.jsx)(TextAreaCode, {
                                part: "textarea",
                                slot: "leader",
                                ref: $.ref.textarea,
                                comments: comments,
                                oninput: onInput,
                                tabSize: tabSize,
                                tabStyle: tabStyle,
                                theme: theme,
                                value: value
                            }),
                            /*#__PURE__*/ (0, _jsxRuntime.jsxs)("div", {
                                slot: "follower",
                                children: [
                                    /*#__PURE__*/ (0, _jsxRuntime.jsx)(CodeSyntax, {
                                        part: "syntax",
                                        slot: "follower",
                                        language: language,
                                        syntax: syntax,
                                        theme: theme,
                                        children: value
                                    }),
                                    textShadow && /*#__PURE__*/ (0, _jsxRuntime.jsx)(Shadow, {}),
                                    /*#__PURE__*/ (0, _jsxRuntime.jsx)(Lenses, {}),
                                    /*#__PURE__*/ (0, _jsxRuntime.jsx)(Markers, {}),
                                    /*#__PURE__*/ (0, _jsxRuntime.jsx)(Caret, {})
                                ]
                            })
                        ]
                    })
                ]
            }));
        $.render(({ fontSize  })=>/*#__PURE__*/ (0, _jsxRuntime.jsxs)(_jsxRuntime.Fragment, {
                children: [
                    /*#__PURE__*/ (0, _jsxRuntime.jsx)("style", {
                        children: style({
                            fontSize
                        })
                    }),
                    /*#__PURE__*/ (0, _jsxRuntime.jsx)("slot", {}),
                    /*#__PURE__*/ (0, _jsxRuntime.jsx)(Content, {})
                ]
            }));
    }
};
__decorate([
    _sigl.default.attr.out()
], CodeEditElement.prototype, "value", void 0);
__decorate([
    _sigl.default.attr.out()
], CodeEditElement.prototype, "fontSize", void 0);
__decorate([
    _sigl.default.attr()
], CodeEditElement.prototype, "autoFocus", void 0);
__decorate([
    _sigl.default.attr()
], CodeEditElement.prototype, "autoResize", void 0);
__decorate([
    _sigl.default.attr()
], CodeEditElement.prototype, "language", void 0);
__decorate([
    _sigl.default.attr()
], CodeEditElement.prototype, "theme", void 0);
__decorate([
    _sigl.default.attr()
], CodeEditElement.prototype, "tabSize", void 0);
__decorate([
    _sigl.default.attr(),
    __metadata("design:type", String)
], CodeEditElement.prototype, "tabStyle", void 0);
__decorate([
    _sigl.default.attr()
], CodeEditElement.prototype, "comments", void 0);
__decorate([
    _sigl.default.attr()
], CodeEditElement.prototype, "textShadow", void 0);
__decorate([
    _sigl.default.attr()
], CodeEditElement.prototype, "hasFocus", void 0);
__decorate([
    _sigl.default.attr(),
    __metadata("design:type", String)
], CodeEditElement.prototype, "caretShift", void 0);
__decorate([
    _sigl.default.attr(),
    __metadata("design:type", String)
], CodeEditElement.prototype, "caretBlink", void 0);
CodeEditElement = __decorate([
    _sigl.default.element()
], CodeEditElement);

//# sourceMappingURL=code-edit.js.map