/** @jsxImportSource sigl */
import $ from 'sigl';
import { CodeSyntaxElement, languages, SyntaxOrImport } from 'code-syntax';
import { SuperImposeElement } from 'super-impose';
import { TextAreaCodeElement } from 'textarea-code';
export { languages };
export type { SyntaxOrImport };
declare global {
    namespace JSX {
        interface IntrinsicAttributes {
            theme?: string;
        }
    }
}
export declare type Marker = {
    key: string;
    index: number;
    size: number;
    source: string;
    class: string;
    el?: HTMLSpanElement;
    rect?: DOMRect;
    oncreate?: (el: HTMLSpanElement) => void;
    onenter?: (el: HTMLSpanElement) => void;
    onleave?: (el: HTMLSpanElement) => void;
    onwheel?: $.EventHandler<any, WheelEvent>;
};
export declare type Lens = {
    line: number;
    message: string;
};
export interface CodeEditElement extends $.Element<CodeEditElement> {
}
/**
 * CodeEdit custom element
 *
 * ```js
 * import { CodeEditElement } from 'code-edit'
 * customElements.define('code-edit', CodeEditElement)
 * ```
 *
 * ```html
 * <code-edit autofocus autoresize language="js" theme="deep"> ... </code-edit>
 * ```
 */
export declare class CodeEditElement extends HTMLElement {
    /** The editor's value */
    value: string;
    /** Font size */
    fontSize: number;
    /** Autofocus to the element when page loads */
    autoFocus: boolean;
    /** Whether to autoresize to the height of the contents */
    autoResize: boolean;
    /** Language for syntax highlighting */
    language: string;
    /** Theme to use */
    theme: string;
    /** Tab size */
    tabSize: number;
    /** Tab style */
    tabStyle: 'spaces' | 'tabs';
    /** Comment */
    comments: string;
    /** Shadow */
    textShadow: boolean;
    hasFocus: boolean;
    CodeSyntax: import("sigl").Component<CodeSyntaxElement, HTMLElement>;
    SuperImpose: import("sigl").Component<SuperImposeElement, HTMLElement>;
    TextAreaCode: import("sigl").Component<TextAreaCodeElement, HTMLElement>;
    TextShadow?: (props: {
        x: number;
        y: number;
    }) => JSX.Element;
    /** Passing a syntax definition regexp manually */
    syntax: SyntaxOrImport;
    textarea?: TextAreaCodeElement;
    layout?: SuperImposeElement;
    height?: HTMLPreElement;
    /** Custom markers */
    markers: Marker[];
    markersCss?: (selector: string) => string;
    hoveringMarker?: Marker | null;
    /** Lens */
    lenses: Lens[];
    root: ShadowRoot;
    get scrollLeft(): number;
    get scrollTop(): number;
    onInput: () => void;
    onScroll: () => void;
    onWheel: $.EventHandler<SuperImposeElement, WheelEvent>;
    caretIndex: number;
    caretShift: 'forward' | 'backward' | 'none';
    caretBlink: 'blink' | 'none';
    caretBlinkTimeout: any;
    updateCaret: () => void;
    mounted($: CodeEditElement['$']): void;
}
